@@ -26,9 +26,21 @@ params = {'legend.fontsize': 'x-large',
          'xtick.labelsize':'x-large',
          'ytick.labelsize':'x-large'}
  
- 
-def test_for_textProgBar():
-    print('test_for_textProgBar')
+def test_PDF2Uniform():
+    inVec = np.random.randn(20000)
+    inds = RobustGaussianFittingLibrary.misc.PDF2Uniform(inVec, 
+                                    numBins=40, nUniPoints = 2000, 
+                                    lowPercentile = 50, highPercentile=100)
+    b, e = np.histogram(inVec, 100)
+    e = e[:-1]
+    b2, e2 = np.histogram(inVec[inds], 100)
+    e2 = e2[:-1]
+    plt.plot(e, b/b.sum())
+    plt.plot(e2, b2/b2.sum())
+    plt.show()
+  
+def test_textProgBar():
+    print('test_textProgBar')
     pBar = RobustGaussianFittingLibrary.misc.textProgBar(180)
     for _ in range(60):
         for _ in range(10000000):
@@ -499,6 +511,7 @@ def test_fitValueSmallSample():
     
 if __name__ == '__main__':
     print('PID ->' + str(os.getpid()))
+    test_PDF2Uniform()
     test_fitBackgroundTensor()
     test_fitBackgroundTensor_multiproc()
     test_RobustAlgebraicLineFittingPy()
@@ -515,4 +528,4 @@ if __name__ == '__main__':
     test_fitValueTensor_MultiProc()
     test_fitLineTensor_MultiProc()
     visOrderStat()
-    exit()
\ No newline at end of file
+    exit()
